#---------Loading data and BIOMOD package---------#
library(BIOMOD)


Dir <- "../CC"
#Dir <- "C:/Users/perca2/Desktop/CC"
Iter <- "1-10"

#load data
Sp.Env <- read.delim(paste(Dir,"/Data/SpeciesEnv.txt",sep = ""))
# the XY coordinates of species data
CoorXY <- Sp.Env[,16:17]

# the environmental variables
Env <- Sp.Env[,c(2:15)]
for (i in 1:dim(Env)[2]) {
  Env[,i] = as.numeric(Env[,i])
}
# the future climate simulations
CS = c(
  "SpeciesEnv","GCM2080_0","GCM2080_15","GCM2080_32",
  "GCM2080_38","GCM2080_45","GCM2080_47","GCM2080_52"
)

Sp.Esp <- c("Abbal","Acsac","Beall","Bepap","Pimar")

#-----------------------------------
#--------1.Build the models--------#
#-----------------------------------

#format data
for (w in 1:length(Sp.Esp)) {
  species = Sp.Env[,Sp.Esp[w]]
  dir.create(paste(Dir,"/Outputs/",Sp.Esp[w],sep = ""),showWarnings = F)
  dir.create(paste(Dir,"/Outputs/",Sp.Esp[w],"/",Iter,sep = ""),showWarnings = F)
  
  setwd(paste(Dir,"/Outputs/",Sp.Esp[w],"/",Iter,sep = ""))
  
  Initial.State(
    Response = species,
    Explanatory = Env,
    IndependentResponse = NULL,
    IndependentExplanatory = NULL,
    sp.name = Sp.Esp[w]
  )
  write.table(
    Sp.Env,file = "SpeciesEnv.txt", col.names = T,row.names = F,sep = " "
  )
  
  #Do modelling
  Models(
    GLM = T, TypeGLM = "quad", Test = "AIC", GBM = T,
    No.trees = 2000, GAM = T, Spline = 3, CTA = T,
    CV.tree = 50, ANN = T, CV.ann = 5, SRE = F,
    Perc025 = F, Perc05 = F, MDA = T, MARS = T, RF = T,
    NbRunEval = 10, DataSplit = 70, NbRepPA = 0,
    strategy = "sre", coor = NULL, distance = 0,
    nb.absences = NULL, Yweights = NULL, VarImport = 5,
    Roc = T, Optimized.Threshold.Roc = T, Kappa = F,
    TSS = F, KeepPredIndependent = F
  )
}
#-----------------------------------
#------2.Project the models--------#
#-----------------------------------

#Projections names
Proj.name = c(
  "Current","GCM2080_0","GCM2080_15","GCM2080_32",
  "GCM2080_38","GCM2080_45","GCM2080_47","GCM2080_52"
)
#Variables names
Variables = c(
  "Tavg_an","PrecTot_an","pratio","savg_elev","lgrpdep_1",
  "lgrpdep_23","lgrpdep_456","lgrpdep_7","lgrpdep_8",
  "lgrpdep_9","lgrpdep_R","lreghy_Mes","lreghy_Xer","lreghy_Hum"
)

for (i in 1:length(Sp.Esp)) {
  setwd(paste(Dir,"/Outputs/",Sp.Esp[i],"/",Iter,sep = ""))
  load(paste(Sp.Esp[i],"_run.RData",sep = ""))
  for (j in 1:length(Proj.name)) {
    tab <- read.delim(paste(Dir,"/Data/", CS[j],".txt",sep = ""))
    Env <- tab[,Variables]
    for (k in 1:dim(Env)[2]) {
      Env[,k] = as.numeric(Env[,k])
    }
    Projection(
      Proj = Env, Proj.name = Proj.name[j],
      GLM = T, GBM = T, GAM = T, CTA = T,
      ANN = T, SRE = F, Perc025 = F,Perc05 = F, MDA = T, MARS = T, RF = T, BinRoc = T,
      BinKappa = F, BinTSS = F, FiltRoc = T, FiltKappa = F, FiltTSS = F, repetition.models = T
    )
  }
}

#-----------------------------------
#------3.Consensus-----------------#
#-----------------------------------

source(paste(Dir,"/Scripts/Auxiliary_Functions.R",sep = ""))
Iter <- "1-10"
Spp.Esp <- c("Abbal","Acsac","Beall","Bepap","Pimar")
dataWithObs <-
  read.delim(paste(Dir,"/Data/SpeciesEnv.txt",sep = ""))
Scenario.Current = "Current"
Scenarios.2080 = c(
  "GCM2080_0","GCM2080_15","GCM2080_32","GCM2080_38",
  "GCM2080_45","GCM2080_47","GCM2080_52"
)
Scenario = list()
Scenario[[1]] = Scenario.Current
Scenario[[2]] = Scenarios.2080
Time = c("Current", "2080")
for (i in 1:length(Spp.Esp)) {
  dataAll <- dataWithObs[,c("Grid_IDKey",Spp.Esp[i])]
  for (j in 1:length(Time)) {
    setwd(paste(Dir,"/Outputs/",Spp.Esp[i],"/",Iter,sep = ""))
    Ens.Forecasting(
      Proj.name = Scenario[[j]],
      Espece = Spp.Esp[i], data.Modl = dataAll,
      data.Proj = dataAll, Time = Time[j]
    )
  }
}

#-----------------------------------
#------4.Evaluation Informations---#
#-----------------------------------

source(paste(Dir,"/Scripts/Auxiliary_Functions.R",sep = ""))
Spp <- c("Abbal","Acsac","Beall","Bepap","Pimar")

setwd(paste(Dir,"/Outputs",sep = ""))
Evaluation.Info()


